@@ -23,23 +23,24 @@ import datetime as dt
 from calendar import timegm
 import dbus, dbus.mainloop.glib
 import gobject
+from utils import stuff, trophies
+
 
 
 def from_dbus_fact(fact):
     """unpack the struct into a proper dict"""
-    return dict(id = fact[0],
-                start_time  = dt.datetime.utcfromtimestamp(fact[1]),
-                end_time = dt.datetime.utcfromtimestamp(fact[2]) if fact[2] else None,
-                description = fact[3],
-                name = fact[4],
-                activity_id = fact[5],
-                category = fact[6],
-                tags = fact[7],
-                date = dt.datetime.utcfromtimestamp(fact[8]).date(),
-                delta = dt.timedelta(days = fact[9] // (24 * 60 * 60),
-                                     seconds = fact[9] % (24 * 60 * 60))
-               )
-
+    return stuff.Fact(fact[4],
+                      start_time  = dt.datetime.utcfromtimestamp(fact[1]),
+                      end_time = dt.datetime.utcfromtimestamp(fact[2]) if fact[2] else None,
+                      description = fact[3],
+                      activity_id = fact[5],
+                      category = fact[6],
+                      tags = fact[7],
+                      date = dt.datetime.utcfromtimestamp(fact[8]).date(),
+                      delta = dt.timedelta(days = fact[9] // (24 * 60 * 60),
+                                           seconds = fact[9] % (24 * 60 * 60)),
+            id = fact[0]
+            )
 
 class Storage(gobject.GObject):
     """Hamster client class, communicating to hamster storage daemon via d-bus.
@@ -161,33 +162,34 @@ class Storage(gobject.GObject):
         """returns fact by it's ID"""
         return from_dbus_fact(self.conn.GetFact(id))
 
-    def add_fact(self, activity_name, tags = '', start_time = None, end_time = None,
-                                      category_name = None, description = None,
-                                      temporary = False):
-        """Add fact. activity name can use `[-]start_time[-end_time] activity@category, description #tag1 #tag2`
+    def add_fact(self, fact, temporary_activity = False):
+        """Add fact. activity name can use the
+        `[-]start_time[-end_time] activity@category, description #tag1 #tag2`
         syntax, or params can be stated explicitly.
-        Params, except for the start and end times will take precedence over
-        derived values.
+        Params will take precedence over the derived values.
         start_time defaults to current moment.
         """
+        if not fact.activity:
+            return None
 
+        serialized = fact.serialized_name()
 
-        start_time = start_time or 0
-        if start_time:
-            start_time = timegm(start_time.timetuple())
+        start_timestamp = timegm((fact.start_time or dt.datetime.now()).timetuple())
 
-        end_time = end_time or 0
-        if end_time:
-            end_time = timegm(end_time.timetuple())
+        end_timestamp = fact.end_time or 0
+        if end_timestamp:
+            end_timestamp = timegm(end_timestamp.timetuple())
 
-        if isinstance(tags, list): #make sure we send what storage expects
-            tags = ", ".join(tags)
-        tags = tags or ''
+        new_id = self.conn.AddFact(serialized,
+                                   start_timestamp,
+                                   end_timestamp,
+                                   temporary_activity)
 
-        category_name = category_name or ''
-        description = description or ''
-
-        return self.conn.AddFact(activity_name, tags, start_time, end_time, category_name, description, temporary)
+        # TODO - the parsing should happen just once and preferably here
+        # we should feed (serialized_activity, start_time, end_time) into AddFact and others
+        if new_id:
+            trophies.checker.check_fact_based(fact)
+        return new_id
 
     def stop_tracking(self, end_time = None):
         """Stop tracking current activity. end_time can be passed in if the
@@ -199,32 +201,27 @@ class Storage(gobject.GObject):
         "delete fact from database"
         self.conn.RemoveFact(fact_id)
 
-    def update_fact(self, fact_id, activity_name, tags = None,
-                    start_time = None, end_time = None,
-                    category_name = None, description = None, temporary = False):
+    def update_fact(self, fact_id, fact, temporary_activity = False):
         """Update fact values. See add_fact for rules.
         Update is performed via remove/insert, so the
         fact_id after update should not be used anymore. Instead use the ID
         from the fact dict that is returned by this function"""
 
-        category_name = category_name or '' # so to override None
-        description = description or '' # so to override None
 
-        start_time = start_time or 0
-        if start_time:
-            start_time = timegm(start_time.timetuple())
+        start_time = timegm((fact.start_time or dt.datetime.now()).timetuple())
 
-        end_time = end_time or 0
+        end_time = fact.end_time or 0
         if end_time:
             end_time = timegm(end_time.timetuple())
 
-        if tags and isinstance(tags, list):
-            tags = ", ".join(tags)
-        tags = tags or ''
+        new_id =  self.conn.UpdateFact(fact_id,
+                                       fact.serialized_name(),
+                                       start_time,
+                                       end_time,
+                                       temporary_activity)
 
-        return self.conn.UpdateFact(fact_id, activity_name, tags,
-                                    start_time, end_time,
-                                    category_name, description, temporary)
+        trophies.checker.check_update_based(fact_id, new_id, fact)
+        return new_id
 
 
     def get_category_activities(self, category_id = None):
