@@ -35,13 +35,13 @@ except ImportError:
 import os, time
 import datetime
 import storage
-import stuff
 from shutil import copy as copyfile
+import itertools
 import datetime as dt
 import gio
 from xdg.BaseDirectory import xdg_data_home
 
-import itertools
+from utils import stuff, trophies
 
 class Storage(storage.Storage):
     con = None # Connection will be created on demand
@@ -73,6 +73,8 @@ class Storage(storage.Storage):
                 print "DB file has been modified externally. Calling all stations"
                 self.dispatch_overwrite()
 
+                # plan "b" – synchronize the time tracker's database from external source while the tracker is running
+                trophies.unlock("plan_b")
 
 
         self.__database_file = gio.File(self.db_path)
@@ -388,13 +390,13 @@ class Storage(storage.Storage):
             self.execute(query, (end_time, fact['id']))
 
     def __squeeze_in(self, start_time):
-        # tries to put task in the given date
-        # if there are conflicts, we will only truncate the ongoing task
-        # and replace it's end part with our activity
+        """ tries to put task in the given date
+            if there are conflicts, we will only truncate the ongoing task
+            and replace it's end part with our activity """
 
         # we are checking if our start time is in the middle of anything
         # or maybe there is something after us - so we know to adjust end time
-        # in the latter case go only few days ahead. everything else is madness, heh
+        # in the latter case go only few hours ahead. everything else is madness, heh
         query = """
                    SELECT a.*, b.name
                      FROM facts a
@@ -405,15 +407,11 @@ class Storage(storage.Storage):
                  ORDER BY start_time
                     LIMIT 1
                 """
-        fact = self.fetchone(query, (start_time,
-                                     start_time,
-                                     start_time - dt.timedelta(seconds = 60 * 60 * 12),
-                                     start_time,
-                                     start_time,
-                                     start_time + dt.timedelta(seconds = 60 * 60 * 12)))
-
+        fact = self.fetchone(query, (start_time, start_time,
+                                     start_time - dt.timedelta(hours = 12),
+                                     start_time, start_time,
+                                     start_time + dt.timedelta(hours = 12)))
         end_time = None
-
         if fact:
             if start_time > fact["start_time"]:
                 #we are in middle of a fact - truncate it to our start
@@ -427,28 +425,29 @@ class Storage(storage.Storage):
 
     def __solve_overlaps(self, start_time, end_time):
         """finds facts that happen in given interval and shifts them to
-        make room for new fact"""
-
-        # this function is destructive - can't go with a wildcard
-        if not end_time or not start_time:
+        make room for new fact
+        """
+        if end_time is None or start_time is None:
             return
 
-        # activities that we are overlapping.
-        # second OR clause is for elimination - |new fact--|---old-fact--|--new fact|
+        # possible combinations and the OR clauses that catch them
+        # (the side of the number marks if it catches the end or start time)
+        #             |----------------- NEW -----------------|
+        #      |--- old --- 1|   |2 --- old --- 1|   |2 --- old ---|
+        # |3 -----------------------  big old   ------------------------ 3|
         query = """
                    SELECT a.*, b.name, c.name as category
                      FROM facts a
                 LEFT JOIN activities b on b.id = a.activity_id
                 LEFT JOIN categories c on b.category_id = c.id
-                    WHERE ((start_time < ? and end_time > ?)
-                           OR (start_time < ? and end_time > ?))
-
-                       OR ((start_time < ? and start_time > ?)
-                           OR (end_time < ? and end_time > ?))
+                    WHERE (end_time > ? and end_time < ?)
+                       OR (start_time > ? and start_time < ?)
+                       OR (start_time < ? and end_time > ?)
                  ORDER BY start_time
                 """
-        conflicts = self.fetchall(query, (start_time, start_time, end_time, end_time,
-                                          end_time, start_time, end_time, start_time))
+        conflicts = self.fetchall(query, (start_time, end_time,
+                                          start_time, end_time,
+                                          start_time, end_time))
 
         for fact in conflicts:
             # won't eliminate as it is better to have overlapping entries than loosing data
@@ -460,22 +459,28 @@ class Storage(storage.Storage):
                fact["start_time"] < end_time < fact["end_time"]:
 
                 logging.info("splitting %s" % fact["name"])
+                # truncate until beginning of the new entry
                 self.execute("""UPDATE facts
                                    SET end_time = ?
                                  WHERE id = ?""", (start_time, fact["id"]))
                 fact_name = fact["name"]
-                new_fact_id = self.__add_fact(fact["name"],
-                                           "", # will create tags in the next step
-                                           end_time,
-                                           fact["end_time"],
-                                           fact["category"],
-                                           fact["description"])
+
+                # create new fact for the end
+                new_fact = stuff.Fact(fact["name"],
+                    category = fact["category"],
+                    description = fact["description"],
+                )
+                new_fact_id = self.__add_fact(new_fact.serialized_name(), end_time, fact["end_time"])
+
+                # copy tags
                 tag_update = """INSERT INTO fact_tags(fact_id, tag_id)
                                      SELECT ?, tag_id
                                        FROM fact_tags
                                       WHERE fact_id = ?"""
                 self.execute(tag_update, (new_fact_id, fact["id"])) #clone tags
 
+                trophies.unlock("split")
+
             # overlap start
             elif start_time < fact["start_time"] < end_time:
                 logging.info("Overlapping start of %s" % fact["name"])
@@ -489,36 +494,19 @@ class Storage(storage.Storage):
                              (start_time, fact["id"]))
 
 
-    def __add_fact(self, activity_name, tags, start_time = None,
-                   end_time = None, category_name = None,
-                   description = None, temporary = False):
+    def __add_fact(self, serialized_fact, start_time, end_time = None, temporary = False):
+        fact = stuff.Fact(serialized_fact,
+                          start_time = start_time,
+                          end_time = end_time)
 
-        activity = stuff.parse_activity_input(activity_name)
-
-        # make sure that we do have an activity name after parsing
-        if not activity.activity_name:
+        if not fact.activity or start_time is None:  # sanity check
             return 0
 
-        # explicitly stated takes precedence
-        activity.description = description or activity.description
 
-        tags = [tag.strip() for tag in tags.split(",") if tag.strip()]  # split by comma
-        tags = tags or activity.tags # explicitly stated tags take priority
+        # get tags from database - this will create any missing tags too
+        tags = [dict(zip(('id', 'name', 'autocomplete'), row))
+                                           for row in self.GetTagIds(fact.tags)]
 
-        tags = [dict(zip(('id', 'name', 'autocomplete'), row)) for row in self.GetTagIds(tags)] #this will create any missing tags too
-
-
-        if category_name:
-            activity.category_name = category_name
-        if description:
-            activity.description = description #override
-
-        start_time = activity.start_time or start_time or datetime.datetime.now()
-
-        start_time = start_time.replace(microsecond = 0)
-        end_time = activity.end_time or end_time
-        if end_time:
-            end_time = end_time.replace(microsecond = 0)
 
         now = datetime.datetime.now()
         # if in future - roll back to past
@@ -533,29 +521,27 @@ class Storage(storage.Storage):
                 end_time -= dt.timedelta(days = 1)
 
 
-        if not start_time or not activity.activity_name:  # sanity check
-            return 0
-
         # now check if maybe there is also a category
         category_id = None
-        if activity.category_name:
-            category_id = self.__get_category_id(activity.category_name)
+        if fact.category:
+            category_id = self.__get_category_id(fact.category)
             if not category_id:
-                category_id = self.__add_category(activity.category_name)
+                category_id = self.__add_category(fact.category)
+
+                trophies.unlock("no_hands")
 
         # try to find activity, resurrect if not temporary
-        activity_id = self.__get_activity_by_name(activity.activity_name,
+        activity_id = self.__get_activity_by_name(fact.activity,
                                                   category_id,
                                                   resurrect = not temporary)
         if not activity_id:
-            activity_id = self.__add_activity(activity.activity_name,
+            activity_id = self.__add_activity(fact.activity,
                                               category_id, temporary)
         else:
             activity_id = activity_id['id']
 
         # if we are working on +/- current day - check the last_activity
         if (dt.datetime.now() - start_time <= dt.timedelta(days=1)):
-
             # pull in previous facts
             facts = self.__get_todays_facts()
 
@@ -566,14 +552,14 @@ class Storage(storage.Storage):
             if previous and previous['start_time'] < start_time:
                 # check if maybe that is the same one, in that case no need to restart
                 if previous["activity_id"] == activity_id \
-                   and previous["tags"] == sorted([tag["name"] for tag in tags]) \
-                   and (previous["description"] or "") == (description or ""):
-                    return previous["id"]
+                   and set(previous["tags"]) == set([tag["name"] for tag in tags]) \
+                   and (previous["description"] or "") == (fact.description or ""):
+                    return None
 
-                # otherwise, if no description is added
-                # see if maybe it is too short to qualify as an activity
+                # if no description is added
+                # see if maybe previous was too short to qualify as an activity
                 if not previous["description"] \
-                    and 60 >= (start_time - previous['start_time']).seconds >= 0:
+                   and 60 >= (start_time - previous['start_time']).seconds >= 0:
                     self.__remove_fact(previous['id'])
 
                     # now that we removed the previous one, see if maybe the one
@@ -582,8 +568,8 @@ class Storage(storage.Storage):
                     if len(facts) > 1 and 60 >= (start_time - facts[-2]['end_time']).seconds >= 0:
                         before = facts[-2]
                         if before["activity_id"] == activity_id \
-                           and before["tags"] == sorted([tag["name"] for tag in tags]):
-                            # essentially same activity - resume it and return
+                           and set(before["tags"]) == set([tag["name"] for tag in tags]):
+                            # resume and return
                             update = """
                                        UPDATE facts
                                           SET end_time = null
@@ -614,7 +600,7 @@ class Storage(storage.Storage):
                     INSERT INTO facts (activity_id, start_time, end_time, description)
                                VALUES (?, ?, ?, ?)
         """
-        self.execute(insert, (activity_id, start_time, end_time, activity.description))
+        self.execute(insert, (activity_id, start_time, end_time, fact.description))
 
         fact_id = self.__last_insert_rowid()
 
@@ -786,6 +772,10 @@ class Storage(storage.Storage):
         else:
             self.execute("delete from activities where id = ?", (id,))
 
+        # Finished! - deleted an activity with more than 50 facts on it
+        if bound_facts >= 50:
+            trophies.unlock("finished")
+
     def __remove_category(self, id):
         """move all activities to unsorted and remove category"""
 
@@ -947,7 +937,7 @@ class Storage(storage.Storage):
     def end_transaction(self):
         self.__con.commit()
         self.__cur.close()
-        self.__con = None
+        self.__con, self.__cur = None, None
         self.register_modification()
 
     def run_fixtures(self):
@@ -1188,6 +1178,10 @@ class Storage(storage.Storage):
             self.execute("UPDATE version SET version = %d" % current_version)
             print "updated database from version %d to %d" % (version, current_version)
 
+            # oldtimer – database version structure had been performed on startup (thus we know that he has been on at least 2 versions)
+            trophies.unlock("oldtimer")
+
+
         """we start with an empty database and then populate with default
            values. This way defaults can be localized!"""
 
