@@ -21,17 +21,17 @@ import gtk
 import time
 import datetime as dt
 
-""" TODO:
-     * hook into notifications and refresh our days if some evil neighbour edit
-       fact window has dared to edit facts
+""" TODO: hook into notifications and refresh our days if some evil neighbour
+          edit fact window has dared to edit facts
 """
-from configuration import runtime, conf
-import stuff, widgets
+import widgets
+from configuration import runtime, conf, load_ui_file
+from utils import stuff
 
 class CustomFactController:
     def __init__(self,  parent = None, fact_date = None, fact_id = None):
 
-        self._gui = stuff.load_ui_file("edit_activity.ui")
+        self._gui = load_ui_file("edit_activity.ui")
         self.window = self.get_widget('custom_fact_window')
 
         self.parent, self.fact_id = parent, fact_id
@@ -50,20 +50,20 @@ class CustomFactController:
         if fact_id:
             fact = runtime.storage.get_fact(fact_id)
 
-            label = fact['name']
-            if fact['category'] != _("Unsorted"):
-                label += "@%s" %  fact['category']
+            label = fact.activity
+            if fact.category != _("Unsorted"):
+                label += "@%s" %  fact.category
 
             self.new_name.set_text(label)
 
-            self.new_tags.set_text(", ".join(fact['tags']))
+            self.new_tags.set_text(", ".join(fact.tags))
 
 
-            start_date = fact["start_time"]
-            end_date = fact["end_time"]
+            start_date = fact.start_time
+            end_date = fact.end_time
 
             buf = gtk.TextBuffer()
-            buf.set_text(fact["description"] or "")
+            buf.set_text(fact.description or "")
             self.get_widget('description').set_buffer(buf)
 
             self.get_widget("save_button").set_label("gtk-save")
@@ -78,8 +78,8 @@ class CustomFactController:
                 fact_date = dt.date.today()
 
             last_activity = runtime.storage.get_facts(fact_date)
-            if last_activity and last_activity[-1]["end_time"]:
-                start_date = last_activity[-1]["end_time"]
+            if last_activity and last_activity[-1].end_time:
+                start_date = last_activity[-1].end_time
 
                 if fact_date != dt.date.today():
                     end_date = start_date + dt.timedelta(minutes=30)
@@ -190,40 +190,24 @@ class CustomFactController:
 
     def on_save_button_clicked(self, button):
         activity_name, temporary = self.new_name.get_value()
-        parsed = stuff.parse_activity_input(activity_name)
-
-        if not parsed.activity_name:
-            return False
-
-        # explicit takes precedence
-        description = self.figure_description()
-
-        tags = self.new_tags.get_text().decode('utf8')
-
-        start_time = self._get_datetime("start")
 
         if self.get_widget("in_progress").get_active():
             end_time = None
         else:
             end_time = self._get_datetime("end")
 
+        fact = stuff.Fact(activity_name,
+                          description = self.figure_description(),
+                          tags = self.new_tags.get_text().decode('utf8'),
+                          start_time = self._get_datetime("start"),
+                          end_time = end_time)
+        if not fact.activity:
+            return False
+
         if self.fact_id:
-            runtime.storage.update_fact(self.fact_id,
-                                        activity_name,
-                                        tags,
-                                        start_time,
-                                        end_time,
-                                        parsed.category_name,
-                                        description,
-                                        temporary = temporary)
+            runtime.storage.update_fact(self.fact_id, fact, temporary)
         else:
-            runtime.storage.add_fact(activity_name,
-                                     tags,
-                                     start_time,
-                                     end_time,
-                                     parsed.category_name,
-                                     description,
-                                     temporary = temporary)
+            runtime.storage.add_fact(fact, temporary)
 
         self.close_window()
 
