@@ -18,19 +18,21 @@
 
 # You should have received a copy of the GNU General Public License
 # along with Project Hamster.  If not, see <http://www.gnu.org/licenses/>.
-import stuff
 import os, sys
 import datetime as dt
 from xml.dom.minidom import Document
 import csv
-from hamster.i18n import C_
-from hamster.configuration import runtime
 import copy
 import itertools
 import re
-
 from string import Template
 
+from configuration import runtime
+from utils import stuff, trophies
+from utils.i18n import C_
+
+from calendar import timegm
+
 def simple(facts, start_date, end_date, format, path):
     facts = copy.deepcopy(facts) # dont want to do anything bad to the input
     report_path = stuff.locale_from_utf8(path)
@@ -46,6 +48,16 @@ def simple(facts, start_date, end_date, format, path):
 
     writer.write_report(facts)
 
+    # some assembly required - hidden - saved a report for single day
+    if start_date == end_date:
+        trophies.unlock("some_assembly_required")
+
+    # I want this on my desk - generated over 10 different reports
+    if trophies.check("on_my_desk") == False:
+        current = trophies.increment("reports_generated")
+        if current == 10:
+            trophies.unlock("on_my_desk")
+
 
 class ReportWriter(object):
     #a tiny bit better than repeating the code all the time
@@ -56,19 +68,19 @@ class ReportWriter(object):
     def write_report(self, facts):
         try:
             for fact in facts:
-                fact["name"]= fact["name"].encode('utf-8')
-                fact["description"] = (fact["description"] or u"").encode('utf-8')
-                fact["category"] = (fact["category"] or _("Unsorted")).encode('utf-8')
+                fact.activity= fact.activity.encode('utf-8')
+                fact.description = (fact.description or u"").encode('utf-8')
+                fact.category = (fact.category or _("Unsorted")).encode('utf-8')
 
                 if self.datetime_format:
-                    fact["start_time"] = fact["start_time"].strftime(self.datetime_format)
+                    fact.start_time = fact.start_time.strftime(self.datetime_format)
 
-                    if fact["end_time"]:
-                        fact["end_time"] = fact["end_time"].strftime(self.datetime_format)
+                    if fact.end_time:
+                        fact.end_time = fact.end_time.strftime(self.datetime_format)
                     else:
-                        fact["end_time"] = ""
+                        fact.end_time = ""
 
-                fact["tags"] = ", ".join(fact["tags"])
+                fact.tags = ", ".join(fact.tags)
 
                 self._write_fact(self.file, fact)
 
@@ -94,10 +106,10 @@ class ICalWriter(ReportWriter):
 
     def _write_fact(self, file, fact):
         #for now we will skip ongoing facts
-        if not fact["end_time"]: return
+        if not fact.end_time: return
 
-        if fact["category"] == _("Unsorted"):
-            fact["category"] = None
+        if fact.category == _("Unsorted"):
+            fact.category = None
 
         self.file.write("""BEGIN:VEVENT
 CATEGORIES:%(category)s
@@ -133,9 +145,13 @@ class TSVWriter(ReportWriter):
         self.csv_writer.writerow([h.encode('utf-8') for h in headers])
 
     def _write_fact(self, file, fact):
-        self.csv_writer.writerow([fact[key] for key in ["name", "start_time",
-                               "end_time", "delta", "category", "description",
-                               "tags"]])
+        self.csv_writer.writerow([fact.activity,
+                                  fact.start_time,
+                                  fact.end_time,
+                                  fact.delta,
+                                  fact.category,
+                                  fact.description,
+                                  fact.tags])
     def _finish(self, file, facts):
         pass
 
@@ -147,13 +163,13 @@ class XMLWriter(ReportWriter):
 
     def _write_fact(self, file, fact):
         activity = self.doc.createElement("activity")
-        activity.setAttribute("name", fact["name"])
-        activity.setAttribute("start_time", fact["start_time"])
-        activity.setAttribute("end_time", fact["end_time"])
-        activity.setAttribute("duration_minutes", stuff.duration_minutes(fact["delta"]))
-        activity.setAttribute("category", fact["category"])
-        activity.setAttribute("description", fact["description"])
-        activity.setAttribute("tags", fact["tags"])
+        activity.setAttribute("name", fact.activity)
+        activity.setAttribute("start_time", fact.start_time)
+        activity.setAttribute("end_time", fact.end_time)
+        activity.setAttribute("duration_minutes", str(stuff.duration_minutes(fact.delta)))
+        activity.setAttribute("category", fact.category)
+        activity.setAttribute("description", fact.description)
+        activity.setAttribute("tags", fact.tags)
         self.activity_list.appendChild(activity)
 
     def _finish(self, file, facts):
@@ -165,22 +181,24 @@ class XMLWriter(ReportWriter):
 class HTMLWriter(ReportWriter):
     def __init__(self, path, start_date, end_date):
         ReportWriter.__init__(self, path, datetime_format = None)
+        self.start_date, self.end_date = start_date, end_date
 
         dates_dict = stuff.dateDict(start_date, "start_")
         dates_dict.update(stuff.dateDict(end_date, "end_"))
 
         if start_date.year != end_date.year:
-            self.title = _(u"Activity log for %(start_B)s %(start_d)s, %(start_Y)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
+            self.title = _(u"Activity report for %(start_B)s %(start_d)s, %(start_Y)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
         elif start_date.month != end_date.month:
-            self.title = _(u"Activity log for %(start_B)s %(start_d)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
+            self.title = _(u"Activity report for %(start_B)s %(start_d)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
         elif start_date == end_date:
-            self.title = _(u"Activity log for %(start_B)s %(start_d)s, %(start_Y)s") % dates_dict
+            self.title = _(u"Activity report for %(start_B)s %(start_d)s, %(start_Y)s") % dates_dict
         else:
-            self.title = _(u"Activity log for %(start_B)s %(start_d)s – %(end_d)s, %(end_Y)s") % dates_dict
+            self.title = _(u"Activity report for %(start_B)s %(start_d)s – %(end_d)s, %(end_Y)s") % dates_dict
 
 
         # read the template, allow override
-        if os.path.exists(os.path.join(runtime.home_data_dir, "report_template.html")):
+        self.override = os.path.exists(os.path.join(runtime.home_data_dir, "report_template.html"))
+        if self.override:
             template = os.path.join(runtime.home_data_dir, "report_template.html")
         else:
             template = os.path.join(runtime.data_dir, "report_template.html")
@@ -193,7 +211,6 @@ class HTMLWriter(ReportWriter):
         self.fact_row_template = self._extract_template('all_activities')
 
         self.by_date_row_template = self._extract_template('by_date_activity')
-        self.by_date_total_row_template = self._extract_template("by_date_category")
 
         self.by_date_template = self._extract_template('by_date')
 
@@ -214,99 +231,70 @@ class HTMLWriter(ReportWriter):
     def _write_fact(self, report, fact):
         # no having end time is fine
         end_time_str, end_time_iso_str = "", ""
-        if fact["end_time"]:
-            end_time_str = fact["end_time"].strftime('%H:%M')
-            end_time_iso_str = fact["end_time"].isoformat()
+        if fact.end_time:
+            end_time_str = fact.end_time.strftime('%H:%M')
+            end_time_iso_str = fact.end_time.isoformat()
 
         category = ""
-        if fact["category"] != _("Unsorted"): #do not print "unsorted" in list
-            category = fact["category"]
+        if fact.category != _("Unsorted"): #do not print "unsorted" in list
+            category = fact.category
 
 
         data = dict(
-            date = fact["date"].strftime(
+            date = fact.date.strftime(
                    # date column format for each row in HTML report
                    # Using python datetime formatting syntax. See:
                    # http://docs.python.org/library/time.html#time.strftime
                    C_("html report","%b %d, %Y")),
-            date_iso = fact["date"].isoformat(),
-            activity = fact["name"],
+            date_iso = fact.date.isoformat(),
+            activity = fact.activity,
             category = category,
-            tags = fact["tags"],
-            start = fact["start_time"].strftime('%H:%M'),
-            start_iso = fact["start_time"].isoformat(),
+            tags = fact.tags,
+            start = fact.start_time.strftime('%H:%M'),
+            start_iso = fact.start_time.isoformat(),
             end = end_time_str,
             end_iso = end_time_iso_str,
-            duration = stuff.format_duration(fact["delta"]) or "",
-            duration_minutes = "%d" % (stuff.duration_minutes(fact["delta"])),
-            duration_decimal = "%.2f" % (stuff.duration_minutes(fact["delta"]) / 60.0),
-            description = fact["description"] or ""
+            duration = stuff.format_duration(fact.delta) or "",
+            duration_minutes = "%d" % (stuff.duration_minutes(fact.delta)),
+            duration_decimal = "%.2f" % (stuff.duration_minutes(fact.delta) / 60.0),
+            description = fact.description or ""
         )
         self.fact_rows.append(Template(self.fact_row_template).safe_substitute(data))
 
 
     def _finish(self, report, facts):
+
         # group by date
-        name_category = lambda fact: (fact['category'], fact['name'])
-
         by_date = []
-        for date, date_facts in itertools.groupby(facts, lambda fact:fact['date']):
-            by_name = sorted(date_facts, key=name_category)
-
-            by_date_rows = []
-            for (category, activity), ac_facts in itertools.groupby(by_name, name_category):
-                duration = dt.timedelta()
-                for fact in ac_facts:
-                    duration += fact['delta']
-
-
-                by_date_rows.append(Template(self.by_date_row_template).safe_substitute(
-                                    dict(activity = activity,
-                                         category = category,
-                                         duration = stuff.format_duration(duration),
-                                         duration_minutes = "%d" % (stuff.duration_minutes(fact["delta"])),
-                                         duration_decimal = "%.2f" % (stuff.duration_minutes(fact["delta"]) / 60.0),
-                                        )
-                                    ))
-
-            by_date_total_rows = []
-            for category, c_facts in itertools.groupby(by_name, lambda fact:fact['category']):
-                duration = dt.timedelta()
-                for fact in c_facts:
-                    duration += fact['delta']
-
-
-                by_date_total_rows.append(Template(self.by_date_total_row_template).safe_substitute(
-                                          dict(category = category,
-                                               duration = stuff.format_duration(duration),
-                                               duration_minutes = "%d" % (stuff.duration_minutes(fact["delta"])),
-                                               duration_decimal = "%.2f" % (stuff.duration_minutes(fact["delta"]) / 60.0),
-                                              )
-                                          ))
-
-
-            res = Template(self.by_date_template).safe_substitute(
-                           dict(date = fact["date"].strftime(
-                                       # date column format for each row in HTML report
-                                       # Using python datetime formatting syntax. See:
-                                       # http://docs.python.org/library/time.html#time.strftime
-                                       C_("html report","%b %d, %Y")),
-                                by_date_activity_rows = "\n".join(by_date_rows),
-                                by_date_category_rows = "\n".join(by_date_total_rows)
-                           ))
-            by_date.append(res)
+        for date, date_facts in itertools.groupby(facts, lambda fact:fact.date):
+            by_date.append((date, [dict(fact) for fact in date_facts]))
+        by_date = dict(by_date)
 
+        date_facts = []
+        date = self.start_date
+        while date <= self.end_date:
+            str_date = date.strftime(
+                        # date column format for each row in HTML report
+                        # Using python datetime formatting syntax. See:
+                        # http://docs.python.org/library/time.html#time.strftime
+                        C_("html report","%b %d, %Y"))
+            date_facts.append([str_date, by_date.get(date, [])])
+            date += dt.timedelta(days=1)
 
 
         data = dict(
             title = self.title,
+            #grand_total = _("%s hours") % ("%.1f" % (total_duration.seconds / 60.0 / 60 + total_duration.days * 24)),
+
             totals_by_day_title = _("Totals by Day"),
             activity_log_title = _("Activity Log"),
+            totals_title = _("Totals"),
 
-            activity_totals_heading = _("totals by activity"),
-            category_totals_heading = _("totals by category"),
+            activity_totals_heading = _("activities"),
+            category_totals_heading = _("categories"),
+            tag_totals_heading = _("tags"),
 
-            show_prompt = _("Show:"),
+            show_prompt = _("Distinguish:"),
 
             header_date = _("Date"),
             header_activity = _("Activity"),
@@ -321,10 +309,17 @@ class HTMLWriter(ReportWriter):
             show_template = _("Show template"),
             template_instructions = _("You can override it by storing your version in %(home_folder)s") % {'home_folder': runtime.home_data_dir},
 
-            all_activities_rows = "\n".join(self.fact_rows),
-            by_date_rows = "\n".join(by_date)
+            start_date = timegm(self.start_date.timetuple()),
+            end_date = timegm(self.end_date.timetuple()),
+            facts = [dict(fact) for fact in facts],
+            date_facts = date_facts,
+
+            all_activities_rows = "\n".join(self.fact_rows)
         )
         report.write(Template(self.main_template).safe_substitute(data))
 
+        if self.override:
+            # my report is better than your report - overrode and ran the default report
+            trophies.unlock("my_report")
 
         return
